# -*- coding: utf-8 -*-
"""Featured Listings."""

# zope imports
from Products.ATContentTypes import ATCTMessageFactory as ATMF
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.dexterity.content import Item
from plone.supermodel import model
from zope import schema
from zope.interface import implementer

# local import
from ps.plone.mls import (
    _,
    PLONE_4,
    interfaces,
)


if PLONE_4:
    from plone.app.textfield.widget import RichTextFieldWidget


class IFeaturedListings(model.Schema):
    """A list of featured MLS Listings."""

    listing_ids = schema.List(
        description=_(
            u'Add one Listing ID per line for each entry to show up.'
        ),
        title=_(u'MLS Listing IDs'),
        unique=True,
        value_type=schema.TextLine(
            title=_(u'ID'),
        ),
    )

    if PLONE_4:
        directives.widget(body_text=RichTextFieldWidget)
    body_text = RichText(
        required=False,
        title=ATMF(u'label_body_text', default=u'Body Text'),
    )


@implementer(
    IFeaturedListings,
    interfaces.IListingTraversable,
)
class FeaturedListings(Item):
    """A list of featured MLS Listings."""



# -*- coding: utf-8 -*-
"""Propertyshelf MLS Plone Embedding."""

# python imports
import logging

# zope imports
from plone import api as ploneapi
from zope.i18nmessageid import MessageFactory

# local imports
from ps.plone.mls import config


PLONE_4 = '4' <= ploneapi.env.plone_version() < '5'
PLONE_5 = '5' <= ploneapi.env.plone_version() < '6'

logger = logging.getLogger(config.PROJECT_NAME)
_ = MessageFactory('ps.plone.mls')



# -*- coding: utf-8 -*-
"""Custom traversers for MLS Embedding items."""

# python imports
import copy

# zope imports
from ZPublisher.BaseRequest import DefaultPublishTraverse
from plone.memoize.view import memoize
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter, queryMultiAdapter
from zope.interface import implementer
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.browser import (
    IBrowserPublisher,
    IBrowserRequest,
)

# local imports
from ps.plone.mls import api
from ps.plone.mls.browser.developments.collection import (
    DevelopmentCollectionViewlet,
    EXCLUDED_SEARCH_FIELDS,
)
from ps.plone.mls.content import featured
from ps.plone.mls.interfaces import (
    IDevelopmentTraversable,
    IListingTraversable,
)


@implementer(IBrowserPublisher)
class MLSItemTraverser(DefaultPublishTraverse):
    """Custom Traverser for MLS Embedding items.

    The traverser looks for a MLS item id in the traversal stack and
    tries to call the corresponding details view. But before it does so, it
    tries to call all (currently) known traversers.

    It also does a check on the MLS item id. By default this one returns
    ``True``. But a subclass can override it to only return items
    that match a given condition.
    """

    detail_view_name = None
    item_id = 'item_id'

    def _lookup_add_on_traverser(self):
        """Call 3rd party traversers."""
        traverser_class = None
        try:
            from plone.app.imaging.traverse import ImageTraverser
        except ImportError:
            pass
        else:
            traverser_class = ImageTraverser

        try:
            from collective.contentleadimage.extender import LeadImageTraverse
        except ImportError:
            pass
        else:
            if not traverser_class:
                traverser_class = LeadImageTraverse

        return traverser_class

    def check_item(self, item_id):
        """Check if the MLS item with given ID is available."""
        return True

    def publishTraverse(self, request, name):
        """See zope.publisher.interfaces.IPublishTraverse"""
        # Try to deliver the default content views.
        try:
            return super(MLSItemTraverser, self).publishTraverse(
                request, name,
            )
        except (NotFound, AttributeError):
            pass

        traverser_class = self._lookup_add_on_traverser()
        if traverser_class is not None:
            try:
                traverser = traverser_class(self.context, self.request)
                return traverser.publishTraverse(request, name)
            except (NotFound, AttributeError):
                pass
        name = self.pre_lookup(name)

        if not self.check_item(name):
            raise NotFound(self.context, name, request)

        self.post_lookup(name)

        if len(self.request.path) > 0:
            view_name = self.get_view_name_from_request()
        else:
            view_name = self.detail_view_name

        default_view = self.context.getDefaultLayout()

        # Let's call the detail view.
        view = queryMultiAdapter(
            (self.context, request), name=view_name,
        )
        if view is not None:
            return view

        # Deliver the default item view as fallback.
        view = queryMultiAdapter(
            (self.context, request), name=default_view,
        )
        if view is not None:
            return view

        raise NotFound(self.context, name, request)

    def pre_lookup(self, name):
        """Pre lookup hook."""
        return name.split('___')[0]

    def post_lookup(self, name):
        """Post lookup hook."""
        # We store the additional subpath for later reference.
        setattr(self.request, 'subpath', copy.copy(self.request.path))

        # We store the item_id parameter in the request.
        setattr(self.request, self.item_id, name)

    def get_view_name_from_request(self):
        view_name = None
        path = self.request.path
        if len(path) > 0:
            view_name = path.pop(-1)
            if view_name.startswith('@@'):
                view_name = view_name[2:]
        return view_name


@adapter(
    IDevelopmentTraversable,
    IBrowserRequest,
)
class DevelopmentTraverser(MLSItemTraverser):
    """Custom Traverser for Developments.

    See ``MLSItemTraverser`` for details.
    """

    __used_for__ = IDevelopmentTraversable
    detail_view_name = 'development-detail'
    item_id = 'development_id'
    has_development = False

    @memoize
    def check_item(self, item_id):
        """Check if the development ID is available."""
        dcv = DevelopmentCollectionViewlet(self.context, self.request, None)
        portal_state = queryMultiAdapter(
            (self.context, self.request),
            name='plone_portal_state',
        )
        lang = portal_state.language()
        mlsapi = api.get_api(context=self.context, lang=lang)
        params = {'fields': u'id'}
        params.update(dcv.config)
        # params['limit'] = dcv.config.get('limit_developments', 5)
        params['limit'] = 1000
        params = api.prepare_search_params(
            params,
            context=self.context,
            omit=EXCLUDED_SEARCH_FIELDS,
        )
        try:
            result = api.Development.search(mlsapi, params=params)
        except Exception:
            return False

        available_ids = [item.id.value for item in result.get_items()]
        return item_id in available_ids

    def post_lookup(self, item_id):
        """Post lookup hook."""
        super(DevelopmentTraverser, self).post_lookup(item_id)
        portal_state = queryMultiAdapter(
            (self.context, self.request),
            name='plone_portal_state',
        )
        lang = portal_state.language()
        item = api.get_development(
            item_id=item_id,
            context=self.context,
            request=self.request,
            lang=lang,
        )
        if item is not None:
            cache = IAnnotations(self.request)
            cache['ps.plone.mls.development.traversed'] = item
            self.has_development = True

    def get_view_name_from_request(self):
        view_name = None
        allowed_view_names = [
            'listings',
        ]
        path = self.request.path
        if len(path) < 1:
            return view_name

        view_name = path.pop(-1)
        if view_name.startswith('@@'):
            view_name = view_name[2:]

        if view_name in allowed_view_names:
            return view_name

        view_name = self.get_listing_view(view_name, additional=path)

        return view_name

    def get_listing_view(self, listing_id, additional=None):
        """"""
        allowed_view_names = [
            'print-listing',
        ]
        if not self.has_development:
            return

        # Check if the listing belongs to the development
        cache = IAnnotations(self.request)
        item = cache['ps.plone.mls.development.traversed']
        params = {
            'listing_ids': listing_id,
        }
        results, batch = item.listings(params=params)
        available_listings = [
            listing.get('id', {}).get('value', None) for listing in results
        ]
        if listing_id not in available_listings:
                raise NotFound(self.context, listing_id, self.request)

        setattr(self.request, 'listing_id', listing_id)

        while additional:
            name = additional.pop()
            if name in allowed_view_names:
                return name
            else:
                raise NotFound(self.context, name, self.request)
        return 'listing-detail'


@adapter(
    IListingTraversable,
    IBrowserRequest,
)
class ListingTraverser(MLSItemTraverser):
    """Custom Traverser for Listings.

    See ``MLSItemTraverser`` for details.
    """

    __used_for__ = IListingTraversable
    detail_view_name = 'listing-detail'
    item_id = 'listing_id'


@adapter(
    featured.IFeaturedListings,
    IBrowserRequest,
)
class FeaturedListingsTraverser(ListingTraverser):
    """Traverser for featured listings.

    It only allows listing ids which are defined in the context.
    """

    __used_for__ = featured.IFeaturedListings

    def check_item(self, item_id):
        """Check if the listing ID is available."""
        listing_ids = [lid.lower() for lid in self.context.listing_ids]
        return item_id.lower() in listing_ids



# -*- coding: utf-8 -*-
"""Test Layer for ps.plone.mls."""

# python imports
import pkg_resources

# zope imports
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    PLONE_FIXTURE,
)
from plone.testing import Layer, z2


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PA_CONTENTTYPES = False
else:
    HAS_PA_CONTENTTYPES = True


class Fixture(PloneSandboxLayer):
    """Custom Test Layer for ps.plone.mls."""
    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        """Set up Zope for testing."""
        # Load ZCML
        import plone.app.dexterity
        self.loadZCML(package=plone.app.dexterity)

        if HAS_PA_CONTENTTYPES:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)

        import ps.plone.mls
        self.loadZCML(package=ps.plone.mls)

    def setUpPloneSite(self, portal):
        """Set up a Plone site for testing."""
        super(Fixture, self).setUpPloneSite(portal)
        # Install into Plone site using portal_setup
        # self.applyProfile(portal, 'Products.CMFPlone:plone')

        # Plone 5 support
        if HAS_PA_CONTENTTYPES:
            self.applyProfile(portal, 'plone.app.contenttypes:default')

        self.applyProfile(portal, 'ps.plone.mls:default')
        self.applyProfile(portal, 'ps.plone.mls:testfixture')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE, ),
    name='ps.plone.mls:Integration',
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, z2.ZSERVER_FIXTURE),
    name='ps.plone.mls:Functional',
)

ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name='ps.plone.mls:Acceptance')

ROBOT_TESTING = Layer(name='ps.plone.mls:Robot')



# -*- coding: utf-8 -*-
"""Interface definitions."""

# zope imports
from zope.interface import Interface


class IListingTraversable(Interface):
    """Marker interface for traversable listings."""


class IDevelopmentTraversable(Interface):
    """Marker interface for traversable listings."""


class IBaseDevelopmentItems(IDevelopmentTraversable):
    """Marker interface for all development 'collection' items."""


class IDevelopmentCollection(IBaseDevelopmentItems):
    """Marker interface for DevelopmentCollection viewlet."""


class IDevelopmentDetails(Interface):
    """Marker interface for DevelopmentDetails view."""


class IDevelopmentListings(Interface):
    """Marker interface for DevelopmentListings view."""


class IPossibleDevelopmentCollection(Interface):
    """Marker interface for possible DevelopmentCollection viewlet."""


class IPossibleListingSearchBanner(Interface):
    """Marker interface for possible Listing Search Banner viewlet."""


class IListingSearchBanner(Interface):
    """Marker interface for Listing Search Banner viewlet."""



# -*- coding: utf-8 -*-
"""Test UI with robot framework."""

# python imports
import os
import robotsuite
try:
    import unittest2 as unittest
except ImportError:
    import unittest


# zope imports
from plone import api
from plone.testing import layered

# local imports
from ps.plone.mls import testing


def test_suite():
    suite = unittest.TestSuite()
    no_robot = 'NO_ROBOT' in os.environ.keys()
    if no_robot or api.env.plone_version() < '4.2':
        # No robot tests for Plone 4.1.x
        return suite

    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [
        os.path.join('robot', doc)
        for doc in os.listdir(robot_dir)
        if doc.startswith('test') and doc.endswith('.robot')
    ]
    for test in robot_tests:
        suite.addTests([
            layered(
                robotsuite.RobotTestSuite(test),
                layer=testing.ROBOT_TESTING,
            ),
        ])
    return suite



# -*- coding: utf-8 -*-
"""Listing search banner."""

# python imports
try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode
import copy
import urlparse

# zope imports
from Acquisition import aq_inner
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.formwidget.namedfile.converter import b64decode_file
from plone.formwidget.namedfile.widget import NamedImageFieldWidget
from plone.namedfile.browser import Download
from plone.namedfile.file import NamedImage
from plone.supermodel.directives import fieldset
from plone.z3cform import z2
# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False
from z3c.form import (
    button,
    field,
)
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import StaticWidgetAttribute
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.interface import (
    alsoProvides,
    noLongerProvides,
)
from zope.schema.vocabulary import (
    SimpleTerm,
    SimpleVocabulary,
)

# local imports
from ps.plone.mls import (
    _,
    PLONE_4,
    PLONE_5,
    config,
)
from ps.plone.mls.interfaces import (
    IListingSearchBanner,
    IPossibleListingSearchBanner,
)

FIELDS_SECTION_1 = [
    'section_1_search_target',
    'section_1_title',
    'section_1_categories',
    'section_1_default_category',
    'section_1_hide_categories',
    'section_1_hide_beds',
    'section_1_hide_section',
]
FIELDS_SECTION_2 = [
    'section_2_search_target',
    'section_2_title',
    'section_2_categories',
    'section_2_default_category',
    'section_2_hide_categories',
    'section_2_hide_beds',
    'section_2_hide_section',
]
FIELDS_SECTION_3 = [
    'section_3_search_target',
    'section_3_title',
    'section_3_categories',
    'section_3_default_category',
    'section_3_hide_categories',
    'section_3_hide_beds',
    'section_3_hide_section',
]
FIELDS_SECTION_4 = [
    'section_4_search_target',
    'section_4_title',
    'section_4_categories',
    'section_4_default_category',
    'section_4_hide_categories',
    'section_4_hide_beds',
    'section_4_hide_section',
]
FIELDS_UI = [
    'image',
    'image_url',
]

LABEL_SECTION_SEARCH_PAGE = _(u'Search Target Page')
LABEL_SECTION_TITLE = _(u'Title')
LABEL_SECTION_CATEGORIES = _(u'Categories')
LABEL_SECTION_DEFAULT_CATEGORY = _(u'Default Category')
LABEL_SECTION_HIDE_CATEGORY = _(u'Hide category')
LABEL_SECTION_HIDE_BEDS = _(u'Hide bedrooms')
LABEL_SECTION_HIDE_SECTION = _(u'Hide section')

DESCRIPTION_SECTION_SEARCH_PAGE = _(
    u'Select the activated listing search page which will be used to show '
    u'the results.'
)
DESCRIPTION_SECTION_TITLE = _(u'')
DESCRIPTION_SECTION_CATEGORIES = _(u'')
DESCRIPTION_SECTION_DEFAULT_CATEGORY = _(u'')
DESCRIPTION_SECTION_HIDE_CATEGORY = _(
    u'Hide the category field and use default setting.'
)
DESCRIPTION_SECTION_HIDE_BEDS = _(
    u'Hide the bedrooms field.'
)
DESCRIPTION_SECTION_HIDE_SECTION = _(
    u'Don\'t show this section at all.'
)

DEFAULT_CATEGORIES_1 = (
    u'all:All:listing_type=rs,cs,ll\n'
    u'houses:Houses:listing_type=rs&object_type=house,mobile,multiplex,'
    u'townhouse,freestanding_villa\n'
    u'condos:Condos:listing_type=rs&object_type=apartment,condominium,'
    u'half_duplex\n'
    u'land:Land:listing_type=ll\n'
    u'commercial:Commercial:listing_type=cs&object_type=\n'
)

DEFAULT_CATEGORIES_2 = (
    u'all:All:listing_type=rl,cl\n'
    u'houses:Houses:listing_type=rl&object_type=house,mobile,multiplex,'
    u'townhouse,freestanding_villa\n'
    u'condos:Condos:listing_type=rl&object_type=apartment,condominium,'
    u'half_duplex\n'
    u'commercial:Commercial:listing_type=cl&object_type=\n'
)


class ISectionForm(form.Schema):
    """Section Search Form schema."""

    category = schema.Choice(
        required=True,
        title=_(u'Category'),
        values=['one', 'two'],
    )

    q = schema.TextLine(
        required=False,
        title=_(u'Location, Keywords, Listing ID, ...'),
    )

    beds = schema.Choice(
        required=False,
        title=_(u'Bedrooms'),
        source='ps.plone.mls.listings.min_bedrooms',
    )

    price_min = schema.Int(
        required=False,
        title=_(u'Price (Min)'),
    )

    price_max = schema.Int(
        required=False,
        title=_(u'Price (Max)'),
    )


class SectionForm(form.Form):
    """Section Search Form."""

    fields = field.Fields(ISectionForm)
    ignoreContext = True
    method = 'post'

    def __init__(self, context, request, config=None, prefix=None):
        super(SectionForm, self).__init__(context, request)
        self.config = config
        self.categories = None
        self.category_queries = {}
        if prefix is not None and isinstance(prefix, basestring):
            self.prefix = 'form.{0}.'.format(prefix)

    @button.buttonAndHandler(PMF(u'label_search', default=u'Search'),
                             name='search')
    def handle_search(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        url = '?'.join([
            self.search_target_url,
            self.prepare_query_string(data),
        ])
        self.request.response.redirect(url)

    def _prepare_category_search(self, query):
        result = []
        prefix = 'form.{0}.widgets.'.format(self.search_target_id)
        params = dict(urlparse.parse_qsl(query))
        for key, value in params.items():
            values = value.split(',')
            if len(values) > 1:
                for item in values:
                    result.append(urlencode({
                        prefix + key + ':list': item,
                    }))
            else:
                result.append(urlencode({prefix + key: value}))
        return '&'.join(result)

    def prepare_query_string(self, data=None):
        prefix = 'form.{0}.widgets.'.format(self.search_target_id)
        category_query = ''
        query = {
            'form.{0}.buttons.search'.format(self.search_target_id): '',
        }
        if not data:
            return ''
        beds = data.pop('beds', None)
        if beds:
            query[prefix + 'beds-min'] = beds
            query[prefix + 'beds-max'] = '--MAXVALUE--'
        category = data.pop('category', None)
        if self.config.get('hide_categories', False):
            category = self.config.get('default_category', None)
        if category:
            q = self.category_queries.get(category)
            category_query = self._prepare_category_search(q)
        for key, value in data.items():
            if value is not None:
                query[prefix + key] = value
        return '&'.join([
            urlencode(query),
            category_query,
        ])

    @property
    def search_target_url(self):
        target_uid = self.config.get('search_target', None)
        if not target_uid:
            return
        obj = api.content.get(UID=target_uid)
        return obj.absolute_url()

    @property
    def search_target_id(self):
        target_uid = self.config.get('search_target', None)
        if not target_uid:
            return
        obj = api.content.get(UID=target_uid)
        return obj.id

    def update(self):
        """Update form to match configuration."""
        omitted = []
        self._generate_categories()
        if self.config.get('hide_categories', False) or not self.categories:
            omitted.append('category')
        if self.config.get('hide_beds', False):
            omitted.append('beds')
        self.fields = field.Fields(ISectionForm).omit(*omitted)
        self.update_fields()
        super(SectionForm, self).update()

    def update_fields(self):
        """Update form field configurations."""
        if 'category' in self.fields:
            field = self.fields['category']
            field.field.vocabulary = self.categories
            default = self.config.get('default_category', None)
            if default not in self.category_queries:
                default = list(self.categories)[0].token
            field.field.default = default

    def _generate_categories(self):
        """Return a new categories vocabulary."""
        categories = self.config.get('categories', None)
        if not categories:
            return
        terms = []
        for category in categories.splitlines():
            try:
                key, title, query = category.split(':')
            except ValueError:
                continue
            else:
                terms.append(SimpleTerm(key, key, title))
                self.category_queries[key] = query

        self.categories = SimpleVocabulary(terms)


class SearchBanner(ViewletBase):
    """Listing Search Banner."""

    if PLONE_5:
        index = ViewPageTemplateFile('templates/search_banner_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/search_banner.pt')
    config = None
    section_1 = None
    section_2 = None
    section_3 = None
    section_4 = None

    @property
    def available(self):
        """Check if this viewlet is available for rendering."""
        return IListingSearchBanner.providedBy(self.context) and (
            self.section_1 or self.section_2 or
            self.section_3 or self.section_4
        )

    def get_config(self):
        """Get the configuration data from annotations."""
        annotations = IAnnotations(self.context)
        self.config = copy.deepcopy(
            annotations.get(config.SETTINGS_LISTING_SEARCH_BANNER, {})
        )

    @property
    def image_url(self):
        """Return the image URL for the background image."""
        image = self.config.get('image', None)
        if image is not None:
            filename, data = b64decode_file(image)
            return '{0}/@@listing-search-banner-image/{1}'.format(
                self.context.absolute_url(),
                filename,
            )
        image_url = self.config.get('image_url', None)
        if image_url is not None and image_url != u'http://':
            return image_url

    def map_config(self, section=None):
        """Return a config specific for a section."""
        config = {}
        if section is None:
            return
        if not isinstance(section, basestring):
            return
        for key in self.config.keys():
            if key.startswith(section):
                new_key = key[len(section) + 1:]
                config[new_key] = self.config.get(key)
        return config

    def setup_forms(self):
        """Initialize the section forms."""
        z2.switch_on(self, request_layer=IFormLayer)
        if not self.config.get('section_1_hide_section', False) and \
                self.config.get('section_1_search_target', None):
            self.section_1 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_1'),
                prefix='section_1',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_1, IWrappedForm)
            self.section_1.update()
        if not self.config.get('section_2_hide_section', False) and \
                self.config.get('section_2_search_target', None):
            self.section_2 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_2'),
                prefix='section_2',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_2, IWrappedForm)
            self.section_2.update()
        if not self.config.get('section_3_hide_section', False) and \
                self.config.get('section_3_search_target', None):
            self.section_3 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_3'),
                prefix='section_3',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_3, IWrappedForm)
            self.section_3.update()
        if not self.config.get('section_4_hide_section', False) and \
                self.config.get('section_4_search_target', None):
            self.section_4 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_4'),
                prefix='section_4',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_4, IWrappedForm)
            self.section_4.update()

    def update_config(self):
        if not self.config.get('section_1_title', None):
            self.config['section_1_title'] = u'Section 1'
        if not self.config.get('section_2_title', None):
            self.config['section_2_title'] = u'Section 2'
        if not self.config.get('section_3_title', None):
            self.config['section_3_title'] = u'Section 3'
        if not self.config.get('section_4_title', None):
            self.config['section_4_title'] = u'Section 4'

    def update(self):
        """Prepare view related data."""
        super(SearchBanner, self).update()
        self.get_config()
        self.update_config()
        self.setup_forms()


class ISearchBannerConfiguration(form.Schema):
    """Listing Search Banner configuration form schema."""

    fieldset('section_1', label=_(u'Section 1'), fields=FIELDS_SECTION_1)
    fieldset('section_2', label=_(u'Section 2'), fields=FIELDS_SECTION_2)
    fieldset('section_3', label=_(u'Section 3'), fields=FIELDS_SECTION_3)
    fieldset('section_4', label=_(u'Section 4'), fields=FIELDS_SECTION_4)
    fieldset('ui', label=_(u'UI'), fields=FIELDS_UI)

    form.widget(image=NamedImageFieldWidget)
    image = schema.ASCII(
        description=_(
            u'Upload an image file which will be used as a background for '
            u'your listing banner search.'
        ),
        required=False,
        title=_(u'Banner Image'),
    )

    image_url = schema.TextLine(
        default=u'http://',
        description=_(
            u'Instead of uploading an image, you may enter the URL of an '
            u'image hosted on another server.'
        ),
        max_length=511,
        required=False,
        title=_(u'Banner Image URL'),
    )

    section_1_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_1_title = schema.TextLine(
        default=_(u'Buy'),
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_1_categories', rows=6)
    except TypeError:
        pass
    section_1_categories = schema.Text(
        default=DEFAULT_CATEGORIES_1,
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_1_default_category = schema.TextLine(
        default=u'houses',
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_1_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_1_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_1_hide_section = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_2_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_2_title = schema.TextLine(
        default=_(u'Rent'),
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_2_categories', rows=6)
    except TypeError:
        pass
    section_2_categories = schema.Text(
        default=DEFAULT_CATEGORIES_2,
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_2_default_category = schema.TextLine(
        default=u'houses',
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_2_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_2_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_2_hide_section = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_3_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_3_title = schema.TextLine(
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_3_categories', rows=6)
    except TypeError:
        pass
    section_3_categories = schema.Text(
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_3_default_category = schema.TextLine(
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_3_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_3_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_3_hide_section = schema.Bool(
        default=True,
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_4_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_4_title = schema.TextLine(
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_4_categories', rows=6)
    except TypeError:
        pass
    section_4_categories = schema.Text(
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_4_default_category = schema.TextLine(
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_4_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_4_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_4_hide_section = schema.Bool(
        default=True,
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )


class SearchBannerConfiguration(form.SchemaForm):
    """Listing Search Banner Configuration Form."""

    schema = ISearchBannerConfiguration
    label = _(u'\'Listing Search Banner\' Configuration')
    description = _(
        u'Adjust the behaviour for this \'Listing Search Banner\'.'
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        return annotations.get(
            config.SETTINGS_LISTING_SEARCH_BANNER,
            annotations.setdefault(config.SETTINGS_LISTING_SEARCH_BANNER, {})
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[config.SETTINGS_LISTING_SEARCH_BANNER] = data
        api.portal.show_message(
            message=PMF(u'Changes saved.'),
            request=self.request,
        )


class SearchBannerStatus(object):
    """Return activation/deactivation status of the viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleListingSearchBanner.providedBy(self.context) and \
            not IListingSearchBanner.providedBy(self.context)

    @property
    def active(self):
        return IListingSearchBanner.providedBy(self.context)


class SearchBannerToggle(object):
    """Toggle listing search banner viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IListingSearchBanner.providedBy(self.context):
            # Deactivate search banner viewlet.
            noLongerProvides(self.context, IListingSearchBanner)
            self.context.reindexObject(idxs=['object_provides', ])
            msg = _(u'\'Listing Search Banner\' deactivated.')
        elif IPossibleListingSearchBanner.providedBy(self.context):
            alsoProvides(self.context, IListingSearchBanner)
            self.context.reindexObject(idxs=['object_provides', ])
            msg = _(u'\'Listing Search Banner\' activated.')
        else:
            msg = _(
                u'The \'Listing Search Banner\' does\'t work with '
                u'this content type. Add \'IPossibleListingSearchBanner\' '
                u'to the provided interfaces to enable this feature.'
            )
            msg_type = 'error'

        api.portal.show_message(
            request=self.request,
            message=msg,
            type=msg_type,
        )
        self.request.response.redirect(self.context.absolute_url())


class BannerImage(Download):
    """Banner image download view."""

    def __init__(self, context, request):
        super(BannerImage, self).__init__(context, request)
        self.filename = None
        self.data = None

        image = self.config.get('image', None)
        if image is not None:
            filename, data = b64decode_file(image)
            data = NamedImage(data=data, filename=filename)
            self.data = data
            self.filename = filename
            # self.width, self.height = self.data.getImageSize()

    @property
    def config(self):
        """Get the configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_LISTING_SEARCH_BANNER, {})

    def _getFile(self):
        return self.data


NoValueBedrooms = StaticWidgetAttribute(
    _('Beds'),
    field=ISectionForm['beds'],
)



# -*- coding: utf-8 -*-
"""Configuration options for the Propertyshelf MLS Plone Embedding."""

PROFILE_ID = u'profile-ps.plone.mls'
INSTALL_PROFILE = '{0}:default'.format(PROFILE_ID)
UNINSTALL_PROFILE = '{0}:uninstall'.format(PROFILE_ID)
PROJECT_NAME = 'ps.plone.mls'

#: Configuration key for development collection settings.
SETTINGS_DEVELOPMENT_COLLECTION = 'ps.plone.mls.developmentcollection'

#: Configuration key for listing search banner settings.
SETTINGS_LISTING_SEARCH_BANNER = 'ps.plone.mls.listingsearchbanner'


#: Time items should be cached in ram.
RAM_CACHE_TIME = 60 * 10



# -*- coding: utf-8 -*-
"""Vocabularies for development projects."""

# zope imports
from plone import api
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

# local imports
from ps.plone.mls import _
from ps.plone.mls.interfaces import IDevelopmentCollection


@implementer(IVocabularyFactory)
class AvailableDevelopmentCollections(object):
    """Vocabulary which returns all available development collections."""

    def __call__(self, context):
        brains = api.content.find(
            object_provides=IDevelopmentCollection,
            sort_on='sortable_title',
        )
        items = []
        for brain in brains:
            title = '{0} ({1})'.format(brain.Title, brain.getPath())
            items.append(SimpleTerm(brain.UID, brain.UID, title))
        return SimpleVocabulary(items)

AvailableDevelopmentCollectionsFactory = AvailableDevelopmentCollections()


@implementer(IVocabularyFactory)
class SortOptionsVocabulary(object):
    """Return list of sort options for a development collection."""

    def __call__(self, context):
        items = []
        items.append(SimpleTerm('created', _(u'Creation Date')))
        items.append(SimpleTerm('sortable_title', _(u'Title')))
        return SimpleVocabulary(items)

SortOptionsVocabularyFactory = SortOptionsVocabulary()



# -*- coding: utf-8 -*-
"""MLS development detail view."""

# python imports
from email import message_from_string
from email.utils import (
    formataddr,
    getaddresses,
)
from mls.apiclient import exceptions
import json
import logging
import pkg_resources

# zope imports
from Acquisition import aq_inner
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api as plone_api
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.formwidget.captcha.widget import CaptchaFieldWidget
from plone.formwidget.captcha.validator import (
    CaptchaValidator,
    WrongCaptchaCode,
)
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
try:
    from plone.mls.listing.interfaces import IMLSUISettings
    HAS_UI_SETTINGS = True
except ImportError:
    HAS_UI_SETTINGS = False
from plone.registry.interfaces import IRegistry  # noqa
from plone.z3cform import z2
from z3c.form import (
    button,
    field,
    validator,
)
from z3c.form.interfaces import HIDDEN_MODE, IFormLayer
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import (
    getUtility,
    queryMultiAdapter,
)
from zope.i18n import translate
from zope.interface import (
    alsoProvides,
    implementer,
)

# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False

# local imports
from ps.plone.mls import (
    _,
    PLONE_4,
    PLONE_5,
    api,
    config,
    utils,
)
from ps.plone.mls.interfaces import IDevelopmentDetails


logger = logging.getLogger(config.PROJECT_NAME)

EMAIL_TEMPLATE = _(
    u'development_contact_email',
    default=u'Enquiry from: {name} <{sender_from_address}>\n'
    u'Development URL: {url}\n'
    u'\n'
    u'Message:\n'
    u'{message}'
)


MAP_JS = """
var isTouch = false;
var map;

window.addEventListener('touchmove', function MoveDetector(){{
    isTouch = true;
    window.removeEventListener('touchmove', MoveDetector);
    map = initializeMap();
}});

function loadScript(src, callback) {{
  var script = document.createElement("script");
  script.type = "text/javascript";
  if (callback) {{
    script.onload = callback;
  }}
  document.getElementsByTagName("head")[0].appendChild(script);
  script.src = src;
}}


function loadGoogleMaps(callback) {{
  if (typeof google === 'object' && typeof google.maps === 'object') {{
    callback();
  }} else {{
    loadScript('https://maps.googleapis.com/maps/api/js?key={ak}', callback);
  }}
}}


function initializeMap() {{
    var center = new google.maps.LatLng({lat}, {lng})

    var myOptions = {{
        zoom: {zoom},
        center: center,
        mapTypeId: google.maps.MapTypeId.TERRAIN,
        mapTypeControl: true,
        disableDoubleClickZoom: true,
        overviewMapControl: true,
        streetViewControl: true,
        scrollwheel: false,
        draggable:!isTouch
    }}

    var map = new google.maps.Map(
        document.getElementById('{map_id}'),
        myOptions
    );

    var has_marker = true;
    if(has_marker) {{
        var myLatlng = new google.maps.LatLng({lat}, {lng});
        var marker = new google.maps.Marker({{
            position: myLatlng,
            map: map,
            icon: {icon}
        }});
    }}

    google.maps.event.addDomListener(window, "resize", function() {{
        var center = map.getCenter();
        google.maps.event.trigger(map, "resize");
        map.setCenter(center);
    }});

    return map;
}};

loadGoogleMaps(initializeMap);

"""


class IContactForm(form.Schema):
    """Contact Form schema."""

    sender_from_address = schema.TextLine(
        constraint=utils.validate_email,
        description=PMF(
            u'help_sender_from_address',
            default=u'',
        ),
        required=True,
        title=PMF(u'label_sender_from_address', default=u'E-Mail'),
    )

    name = schema.TextLine(
        description=PMF(
            u'help_sender_fullname',
            default=u'',
        ),
        required=True,
        title=PMF(u'label_name', default=u"Name"),
    )

    phone = schema.TextLine(
        required=False,
        title=PMF(u'label_phone', default=u'Phone')
    )

    message = schema.Text(
        constraint=utils.contains_nuts,
        description=PMF(
            u'help_message',
            default=u'',
        ),
        max_length=1000,
        required=True,
        title=PMF(u'label_message', default=u'Message'),
    )

    form.widget(captcha=CaptchaFieldWidget)
    captcha = schema.TextLine(
        required=True,
        title=_(u'Captcha'),
    )

    subject = schema.TextLine(
        required=False,
        title=PMF(u'label_subject', default=u'Subject')
    )


class ContactForm(form.Form):
    """Contact Form."""

    fields = field.Fields(IContactForm)
    ignoreContext = True
    method = 'post'
    _email_sent = False
    fields['captcha'].widgetFactory = CaptchaFieldWidget
    email_override = None

    def __init__(self, context, request, info=None, development=None):
        super(ContactForm, self).__init__(context, request)
        self.item_info = info
        self.development = development

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    def update(self):
        if self.config.get('show_captcha', False) is False:
            self.fields = field.Fields(IContactForm).omit('captcha')

        email_override = self.config.get('contact_override', None)

        if email_override is not None:
            self.email_override = email_override

        super(ContactForm, self).update()

    def updateWidgets(self):
        super(ContactForm, self).updateWidgets()
        urltool = plone_api.portal.get_tool(name='portal_url')
        portal = urltool.getPortalObject()
        subject = u'{portal_title}: {title}'.format(
            portal_title=portal.getProperty('title').decode('utf-8'),
            title=self.development.title.value,
        )
        self.widgets['subject'].mode = HIDDEN_MODE
        self.widgets['subject'].value = subject

    @property
    def already_sent(self):
        return self._email_sent

    @button.buttonAndHandler(PMF(u'label_send', default='Send'), name='send')
    def handle_send(self, action):
        """Send button for sending the email."""
        can_send = True
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        if 'captcha' in data:
            can_send = False
            # Verify the user input against the captcha
            captcha = CaptchaValidator(
                self.context,
                self.request,
                None,
                IContactForm['captcha'],
                None,
            )
            try:
                can_send = captcha.validate(data['captcha'])
            except WrongCaptchaCode, e:
                self.status = e.doc()
                return

        if not self.already_sent and can_send:
            if self.send_email(data):
                self._email_sent = True
                plone_api.portal.show_message(
                    message=_(u'Your contact request was sent successfully.'),
                    request=self.request,
                    type='info',
                )
        self.request.response.redirect(self.request.URL)
        return

    def send_email(self, data):
        """Construct and send an email message."""
        urltool = plone_api.portal.get_tool(name='portal_url')
        portal = urltool.getPortalObject()

        try:
            email_charset = plone_api.portal.get_registry_record(
                'plone.email_charset',
            )
        except plone_api.exc.InvalidParameterError:
            email_charset = portal.getProperty('email_charset', 'utf-8')

        recipient = None
        portal_address = portal.getProperty('email_from_address')

        try:
            agent = self.item_info
            recipient = agent.email.value
        except Exception:
            recipient = portal_address

        recipients = [recipient]
        bcc = self.config.get('contact_form_bcc', None)
        if bcc is not None:
            recipients += [formataddr(addr) for addr in getaddresses((bcc, ))]

        if self.email_override is not None:
            recipient = self.email_override

        sender = formataddr((data['name'], data['sender_from_address']))

        subject = data['subject']
        data['url'] = self.request.getURL()
        body = translate(
            EMAIL_TEMPLATE,
            context=self.request,
        ).format(**data)
        email_msg = message_from_string(body.encode(email_charset))
        email_msg['To'] = formataddr((recipient, recipient))

        plone_api.portal.send_email(
            sender=sender,
            recipient=recipients,
            subject=subject,
            body=email_msg,
        )
        return True


# Register Captcha validator for the captcha field in the ICaptchaForm
validator.WidgetValidatorDiscriminators(
    CaptchaValidator, field=IContactForm['captcha'])


@implementer(IDevelopmentDetails)
class DevelopmentDetails(BrowserView):
    """Detail view for MLS developments."""

    _item = None
    _contact_form = None
    _contact_info = None

    if PLONE_5:
        index = ViewPageTemplateFile('templates/development_details_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/development_details_view.pt')

    def __call__(self):
        self.setup()
        return self.render()

    def render(self):
        return self.index()

    def setup(self):
        self.registry = getUtility(IRegistry)  # noqa
        if PLONE_5:
            from Products.CMFPlone.resources import add_resource_on_request
            if self.use_fotorama():
                add_resource_on_request(self.request, 'psplonefotorama')
            try:
                pkg_resources.get_distribution('ps.plone.realestatefont')
            except pkg_resources.DistributionNotFound:
                pass
            else:
                from Products.GenericSetup.tool import UNKNOWN
                setup = plone_api.portal.get_tool(name='portal_setup')
                profile = 'profile-ps.plone.realestatefont:default'
                if setup.getLastVersionForProfile(profile) != UNKNOWN:
                    add_resource_on_request(
                        self.request,
                        'psplonerealestatefont',
                    )

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    @property
    def item(self):
        if self._item is None:
            self._item = self._get_item()
        return self._item

    def _get_item(self):
        cache = IAnnotations(self.request)
        item = cache.get('ps.plone.mls.development.traversed', None)

        if item is None:
            item_id = getattr(self.request, 'development_id', None)
            if not item_id:
                return

            portal_state = queryMultiAdapter(
                (self.context, self.request),
                name='plone_portal_state',
            )
            lang = portal_state.language()
            item = api.get_development(
                item_id=item_id,
                context=self.context,
                request=self.request,
                lang=lang,
            )
        return item

    @property
    def lead_image(self):
        """"""
        item = self.item
        try:
            img = item.lead_image
        except AttributeError:
            pass
        else:
            return img

        try:
            images = item.pictures()
        except AttributeError:
            pass
        else:
            if len(images) > 0:
                return images[0]

    @property
    def map_id(self):
        """Generate a unique css id for the map."""
        try:
            item_id = self.item.id.value
        except Exception:
            item_id = 'unknown'
        return u'map__{0}'.format(item_id)

    def javascript_map(self):
        """Return the JS code for the map."""
        if not hasattr(self.item, 'geolocation') or not self.item.geolocation:  # noqa
            return

        icon = getattr(self.item, 'icon', None)
        if icon is not None:
            icon = icon.value
        icon_url = json.dumps(icon)

        lat, lng = self.item.geolocation.value.split(',')

        return MAP_JS.format(
            icon=icon_url,
            lat=lat,
            lng=lng,
            map_id=self.map_id,
            zoom=self.config.get('map_zoom_level', 7),
            ak=self.googleapi,
        )

    @property
    def googleapi(self):
        if not HAS_UI_SETTINGS:
            return ''

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'googleapi', '')
        return ''

    def use_fotorama(self):
        if not HAS_UI_SETTINGS:
            return False

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'slideshow') == u'fotorama'
        return False

    def use_galleria(self):
        if not HAS_UI_SETTINGS:
            return True

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'slideshow') == u'galleria'
        # Fallback: 'galleria' is the default.
        return True

    def titles_for_phases(self):
        """Get the titles for the Development Phase fields."""
        raw = api.DevelopmentPhase.get_field_titles(self.item._api)
        return raw.get('response', {}).get('fields', {})

    def titles_for_groups(self):
        """Get the titles for the Property Group fields."""
        raw = api.PropertyGroup.get_field_titles(self.item._api)
        return raw.get('response', {}).get('fields', {})

    def distance_class(self):
        """Count how many distances are set"""
        counter = 0
        # get all distances
        item = self.item
        airport = item.airport_name or item.airport_distance
        bank = item.bank_name or item.bank_distance
        gas_station = item.gas_station_name or item.gas_station_distance
        hospital = item.hospital_name or item.hospital_distance
        shopping = item.shopping_name or item.shopping_distance

        if airport is not None:
            counter += 1
        if bank is not None:
            counter += 1
        if gas_station is not None:
            counter += 1
        if hospital is not None:
            counter += 1
        if shopping is not None:
            counter += 1

        return 'count_{0}'.format(counter)

    def show_section_contact(self):
        """Should the contact us section be shown at all?"""
        show_form = self.contact_form() is not None
        show_info = self.contact_info() is not None
        return show_info or show_form

    def _get_contact_info(self):
        if self._contact_info is not None:
            return self._contact_info

        item = self.item
        agency = getattr(item, 'agency', None)
        if agency is not None and agency() is not None:
            agency = agency()
            agency.override(context=self.context)
        agent = getattr(item, 'agent', None)
        if agent is not None and agent() is not None:
            agent = agent()
            agent.override(context=self.context)
        self._contact_info = {
            'agency': agency,
            'agent': agent,
        }
        return self._contact_info

    def contact_info(self):
        """Get the contact information, if enabled."""
        if not self.config.get('show_contact_info', False):
            return
        return self._get_contact_info()

    def contact_form(self):
        """Get the contact form, if enabled."""
        if not self.config.get('show_contact_form', False):
            return

        if self._contact_form is not None:
            return self._contact_form

        item_info = self._get_contact_info().get('agent')
        z2.switch_on(self, request_layer=IFormLayer)
        self._contact_form = ContactForm(
            aq_inner(self.context),
            self.request,
            info=item_info,
            development=self.item,
        )
        if HAS_WRAPPED_FORM:
            alsoProvides(self._contact_form, IWrappedForm)
        return self._contact_form

    def contact_link(self):
        return self.config.get('show_contact_link', False)

    def get_field_label(self, field_name):
        """Get the field label for ``field_name``.

        Do this even if the data may not exist within the current
        development object.
        """
        field = api.Field(field_name, None, self.item)
        return field.title

    def format_distance(self, name, distance):
        """Format the distance labels.

        Do this in the form of ``name - distance`` but also correctly handle
        None values.
        """
        text = []
        if name is not None:
            text.append(name)
        if distance is not None:
            text.append(distance)
        return u' - '.join(text)

    def base_url(self):
        return self.context.absolute_url()

    def group_listings(self, group=None):
        """Return the property group listings."""
        try:
            group_id = group.id.value
        except Exception:
            return

        try:
            item = api.PropertyGroup.get(self.item._api, group_id)
        except exceptions.ResourceNotFound:
            return None

        params = {
            'sort_on': 'last_activated_date',
            'reverse': '1',
        }
        try:
            results, batching = item.listings(params=params)
        except exceptions.MLSError, e:
            logger.warn(e)

        if results is None:
            return
        return ListingBatch(results, 0, batch_data=batching)


class HeaderViewlet(ViewletBase):
    """Header Image"""

    _id = None
    _title = None
    _headline = None
    _location = None
    _logo = None
    _banner = None
    _has_banner = False

    @property
    def available(self):
        # do we have a development?
        self._id = getattr(self.request, 'development_id', None)
        if self._id is not None:
            return True
        else:
            return False

    @property
    def get_title(self):
        """Get development title"""
        return self._title

    @property
    def get_headline(self):
        """Get development headline"""
        return self._headline

    @property
    def get_location(self):
        """Get development location"""
        return self._location

    @property
    def get_logo(self):
        """Get development logo"""
        return self._logo

    @property
    def get_banner(self):
        """Get development header"""
        return self._banner

    def update(self):
        """Prepare view related data."""
        super(HeaderViewlet, self).update()

        if self.available:
            self._set_development_info()

    def _set_banner(self, item):
        """Look for available Header image"""
        try:
            # banner image as regular data
            self._banner = item.banner_image.value
            self._has_banner = True
        except Exception:
            self._has_banner = False

    def _set_development_info(self):
        """Set all available data for the development header"""
        cache = IAnnotations(self.request)
        item = cache.get('ps.plone.mls.development.traversed', None)

        if item is not None:
            # try to set the available data
            try:
                self._logo = item.logo.value
            except Exception:
                pass
            try:
                self._title = item.title.value
            except Exception:
                pass
            try:
                self._headline = item.headline.value
            except Exception:
                pass
            try:
                seq = (item.city.value,
                       item.subdivision.value,
                       item.country.value)
                joint = ', '
                self._location = joint.join(seq)
            except Exception:
                self._location = item.location.value
            # set header image
            self._set_banner(item)


class DevelopmentCanonicalURL(ViewletBase):
    """Defines a canonical link relation viewlet.

    This needs to be displayed across the site. A canonical page is
    the preferred version of a set of pages with highly similar
    content. For more information, see:
    https://tools.ietf.org/html/rfc6596
    https://support.google.com/webmasters/answer/139394?hl=en
    """

    @memoize
    def render(self):
        context_state = queryMultiAdapter(
            (self.context, self.request), name=u'plone_context_state')
        base_url = context_state.current_base_url()
        return u'    <link rel="canonical" href="{0}" />'.format(base_url)



# -*- coding: utf-8 -*-
"""Customized plone viewlets."""

# zope imports
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.layout.viewlets import common
from plone.mls.listing.browser.interfaces import IListingDetails

# local imports
from ps.plone.mls.interfaces import (
    IDevelopmentDetails,
    IDevelopmentListings,
)
from ps.plone.mls import (
    _,
    utils,
)


class DublinCoreViewlet(common.DublinCoreViewlet):
    """Customized DublinCore descriptions for MLS embeddings."""

    def _get_dc_tags(self):
        """Generate the Dublin Core meta tags for an embedded item."""
        dc = {}

        if IListingDetails.providedBy(self.view):
            dc['DC.date.modified'] = self.view.data.get('modified')
            dc['DC.date.created'] = self.view.data.get('created')
            dc['DC.creator'] = self._get_mls_creator()
            dc['DC.type'] = u'MLS Listing'

        elif IDevelopmentDetails.providedBy(self.view):
            # Not yet implemented
            dc['DC.date.modified'] = None
            # Not yet implemented
            dc['DC.date.created'] = None
            dc['DC.creator'] = self._get_mls_creator()
            dc['DC.type'] = u'MLS Development'

        return dc

    def _get_mls_creator(self):
        """Get the creator/author from an embedded item."""
        if IListingDetails.providedBy(self.view):
            try:
                contact = self.view.contact
            except AttributeError:
                return
            else:
                return contact.get('agency', {}).get('name', {}).get('value')
        elif IDevelopmentDetails.providedBy(self.view):
            try:
                return self.view.item.agency().title.value
            except AttributeError:
                return

    def _get_mls_description(self):
        """Get the description from an embedded item."""
        description = None

        if IListingDetails.providedBy(self.view):
            try:
                description = self.view.description
            except AttributeError:
                return
        elif IDevelopmentDetails.providedBy(self.view):
            try:
                description = self.view.item.description.value
            except AttributeError:
                return

        description = utils.smart_truncate(description)
        return description

    @property
    def available(self):
        """Check if the preconditions are fullfilled."""
        is_development = IDevelopmentDetails.providedBy(self.view)
        is_listing = IListingDetails.providedBy(self.view)
        return is_development or is_listing

    def update(self):
        super(DublinCoreViewlet, self).update()

        if not self.available:
            return

        try:
            use_all = api.portal.get_registry_record(
                'plone.exposeDCMetaTags'
            )
        except InvalidParameterError:
            try:
                props = api.portal.get_tool(name='portal_properties')
                use_all = props.site_properties.exposeDCMetaTags
            except Exception:
                use_all = False

        meta_dict = dict(self.metatags)
        description = self._get_mls_description()

        if use_all:
            meta_dict['DC.description'] = description
            meta_dict.update(self._get_dc_tags())
        else:
            meta_dict['description'] = description

        self.metatags = meta_dict.items()


class TitleViewlet(common.TitleViewlet):
    """Customized title Viewlet for MLS embeddings."""

    def update(self):
        super(TitleViewlet, self).update()

        title = None
        if IDevelopmentDetails.providedBy(self.view):
            try:
                title = self.view.item.title.value
            except AttributeError:
                title = getattr(self.request, 'development_id', None)
        elif IListingDetails.providedBy(self.view):
            try:
                title = self.view.title
            except AttributeError:
                title = getattr(self.request, 'listing_id', None)
        elif IDevelopmentListings.providedBy(self.view):
            try:
                title = self.view.item.title.value
            except AttributeError:
                title = getattr(self.request, 'development_id', None)
            if title is not None:
                title = u'{0} &mdash; {1}'.format(
                    _(u'Listings'),
                    title,
                )

        if title is not None:
            self.site_title = u'{0} &mdash; {1}'.format(title, self.site_title)



# -*- coding: utf-8 -*-
"""Post install import steps for ps.plone.mls."""

# python imports
import pkg_resources

# zope imports
from Products.CMFPlone.interfaces import INonInstallable
from Products.GenericSetup.interfaces import IProfileImportedEvent
from plone import api
from zope.component import adapter
from zope.interface import implementer

# local imports
from ps.plone.mls import config


ADD_ONS = [
    'ps.plone.fotorama',
]


@implementer(INonInstallable)
class HiddenProfiles(object):
    """Define hidden GenericSetup profiles."""

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            'ps.plone.mls:testfixture',
            'ps.plone.mls:uninstall',
        ]


@adapter(IProfileImportedEvent)
def handle_profile_imported_event(event):
    """Update 'last version for profile' after a full import."""
    if event.profile_id == 'profile-plone.app.upgrade.v50:to50alpha3':
        setup = api.portal.get_tool(name='portal_setup')
        setup.runAllImportStepsFromProfile(config.INSTALL_PROFILE)


def install_add_ons(context):
    """Install additional available add-ons."""
    if not context.readDataFile('plone.mls.listing_various.txt'):
        return

    setup = api.portal.get_tool(name='portal_setup')
    quickinstaller = api.portal.get_tool(name='portal_quickinstaller')
    for item in ADD_ONS:
        try:
            pkg_resources.get_distribution(item)
        except pkg_resources.DistributionNotFound:
            continue
        # Only install add-ons which are not installed yet.
        if not quickinstaller.isProductInstalled(item):
            if quickinstaller.isProductInstallable(item):
                try:
                    profile = 'profile-{0}:default'.format(item)
                    setup.runAllImportStepsFromProfile(profile)
                except AttributeError:
                    pass



# -*- coding: utf-8 -*-
"""Vocabularies for listings."""

# zope imports
from plone import api
from plone.mls.listing.browser.listing_search import IListingSearch
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary


MIN_BEDROOM_VALUES = [
    (1, '1'),
    (2, '2'),
    (3, '3'),
    (4, '4'),
    (5, '5+'),
]


@implementer(IVocabularyFactory)
class AvailableListingSearches(object):
    """Vocabulary which returns all available listing searches."""

    def __call__(self, context):
        brains = api.content.find(
            object_provides=IListingSearch,
            sort_on='sortable_title',
        )
        items = []
        for brain in brains:
            title = '{0} ({1})'.format(brain.Title, brain.getPath())
            items.append(SimpleTerm(brain.UID, brain.UID, title))
        return SimpleVocabulary(items)

AvailableListingSearchesFactory = AvailableListingSearches()


@implementer(IVocabularyFactory)
class MinBedroomsVocabulary(object):
    """Vocabulary which returns the values for the min bedrooms field."""

    def __call__(self, context):
        items = []
        for item in MIN_BEDROOM_VALUES:
            items.append(SimpleTerm(item[0], item[0], item[1]))
        return SimpleVocabulary(items)

MinBedroomsVocabularyFactory = MinBedroomsVocabulary()
